use crate::worker::Worker;
use once_cell::sync::Lazy;
use sha2::{Digest, Sha512};
use std::fs;
use std::path::PathBuf;
use u_lib::utils::{Hexlify, SharedStorage};

pub struct FileMeta {
    hashsum: String,
    path: PathBuf,
    name: String,
}

static FILES: Lazy<SharedStorage<String, FileMeta>> = SharedStorage::new();

pub struct FileIndex;

impl FileIndex {
    pub fn search_by_name(pat: impl Into<String>) -> Option<String> {
        let name = pat.into();
        let (key, _) = FILES.lock().iter().find(|(_, v)| v.name == name)?;
        Some(*key)
    }

    pub fn get(key: impl Into<String>) -> Option<FileMeta> {
        let name = key.into();
        FILES.get(&name).and_then(|r| Some(*r))
    }
}

impl Worker for FileIndex {
    fn process(&mut self) {
        let hexlify = |data: &[u8]| format!("{:x}", Hexlify(data));
        let mut hasher = Sha512::new();
        for file in fs::read_dir("./files/hosted/").unwrap() {
            let file = match file {
                Ok(f) => f.path(),
                Err(e) => {
                    warn!("Error reading file: {}", e);
                    continue;
                }
            };
            if file.is_file() {
                let file_data = fs::read(file).unwrap();
                hasher.update(file_data);
                let hashsum = hasher.finalize_reset();
                let shrinked_sum = hashsum
                    .iter()
                    .cloned()
                    .take(32)
                    .map(|b| b ^ 0x5a)
                    .collect::<Vec<u8>>();
                let shrinked_sum = hexlify(&shrinked_sum);
                let hashsum = hexlify(&hashsum);
                let name = file.file_name().unwrap().to_string_lossy().into_owned();
                let meta = FileMeta {
                    hashsum,
                    name,
                    path: file,
                };
                FILES.lock().insert(shrinked_sum, meta);
            }
        }
    }
}
